// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj02_db.h was generated from the DDL file obj02_db.ddl.
#include "obj02_db.h"

// obj02_db_api.h was generated from the DDL file obj02_db.ddl. with 
// the -cpp option
#include "obj02_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj02 for C++
// Each time you run this example, a new record is created in the
// database, then all existing records are read from the database
// and printed.

// The DDL (Database Definition Language) is in obj02_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_records
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj02_db' in this
// project directory.

int32_t main(void)
{
    Db_obj02_db  db;
    struct obj02 obj02_input;
    struct obj02 obj02_output;
    Cursor_obj02 obj02_cursor;
    int16_t      do_initialize = 0;

    // Open the database and check to see if the db file exists
    try
    {
        db = db.Open("obj02_db");
        obj02_cursor = db.Get_obj02_records();
    }
    catch (rdm_exception& e)
    {
        // This is the error code if the data file does not exist.
        // In this case we will call db->Initialize to create the file.
        if (e.error() == S_NOFILE)
        {
            do_initialize = 1;
        }
        else
        {
            cerr << "Error " << e.error() << " (" << e.what() << ") opening obj02_db database" << endl;
            return 1;
        }
    }

    // Create the database files (if necessary)
    if(do_initialize)
    {
        try
        {
            db.Initialize();
        }
        catch (rdm_exception& e)        
        {
            cerr << "Error " << e.error() << " (" << e.what() << ") initializing obj02_db database" << endl;
            return 1;
        }
    }

    // Create a new record in the database
    try
    {
        strncpy(obj02_input.message, "Hello World", SIZEOF_MESSAGE);
        obj02_cursor = db.New_obj02_recordWithFieldValues(&obj02_input);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding record" << endl;
        return 1;
    }

    // scan and read all created records
    // one additional record will be created each time you run this example
    obj02_cursor = db.Get_obj02_records();
    while (!obj02_cursor.AfterLast())
    {
        obj02_cursor.GetFieldValues(&obj02_output);
        cout << obj02_output.message << endl;
        obj02_cursor++;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return 0;
}
