#ifndef OBJ02_DB_API_H
#define OBJ02_DB_API_H

/* RDM 11.0.0.1000 */

#include "Db.h"
#include "Cursor.h"
#include "obj02_db.h"

namespace RDM_CPP {

class Cursor_obj02;

/*! \brief The obj02_db Db class
 *
 * This class provides the implementation for the obj02_db database
*/
class Db_obj02_db : public Db {
public:
    /*! \brief Open the obj02_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj02_db object
     */
    static Db_obj02_db Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj02_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj02_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj02_db Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the obj02_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj02_db object
     */
    static Db_obj02_db Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj02_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj02_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj02_db Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_obj02_db Default Constructor
    Db_obj02_db (void) throw ();
    /*! \brief Db_obj02_db Constructor
     *
     * \param db A Db object used to create the Db_obj02_db
     */
    Db_obj02_db (const Db& db) throw ();
    //! Db_obj02_db Copy Constructor
    Db_obj02_db (const Db_obj02_db&) throw ();

    //! Db_obj02_db assignment operator
    Db_obj02_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_obj02_db assignment operator
    Db_obj02_db& operator = (const Db_obj02_db&) throw ();

    /*! \brief Db_obj02_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_obj02_db ();

    /*! \brief Create a new obj02 record
     *
     * \returns A singleton Cursor_obj02 object with default field values
     *
     * Create a new obj02 record using default values
     */
    Cursor_obj02 New_obj02_record (void) const throw (rdm_exception&);

    /*! \brief Create a new obj02 record
     *
     * \param fields The field values for the new obj02 record
     * \returns A singleton Cursor_obj02 object with the specified field values
     *
     * Create a new obj02 record using the specified values
     */
    Cursor_obj02 New_obj02_recordWithFieldValues (struct obj02 *fields) const throw (rdm_exception&);

    /*! \brief Obtain obj02 cursor position at the first obj02 record
     * \returns obj02 cursor positioned at the first obj02 record
     *
     * Create a new obj02 cursor for doing a table scan of the obj02 records.
     * The cursor is positioned at the first obj02 record
     */
    Cursor_obj02 Get_obj02_records (void) const throw (rdm_exception&);

private:
    /*! \brief Db_obj02_db reference operator */
    Db_obj02_db* operator & (void);
    /*! \brief Db_obj02_db constructor */
    Db_obj02_db (IDb *idb);
    /*! \brief Get the Db_obj02_db dbType
     * \return The Db_obj02_db dbType
     */
    static dbType Type (void);
};

/*! \brief The obj02 Cursor class
 *
 * This class provides the implementation for the obj02 Cursor
*/
class Cursor_obj02 : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_obj02 objects
     */
    static recordType Type (void);

    /*! Cursor_obj02 Default Constructor */
    Cursor_obj02 (void) throw ();
    /*! \brief Cursor_obj02 Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_obj02 object
     */
    Cursor_obj02 (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_obj02 Copy Constructor */
    Cursor_obj02 (const Cursor_obj02&) throw ();

    /*! \brief Cursor_obj02 assignment operator */
    Cursor_obj02& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_obj02 assignment operator */
    Cursor_obj02& operator = (const Cursor_obj02& cursor) throw ();

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct obj02 *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct obj02 *fields) const throw (rdm_exception&);

    /*! \brief Set the message field value for the current record
     * \param message The field value that will be written to the record at the current cursor position
     */
    void Set_message (const char message_val[32]) const throw (rdm_exception&);

    /*! \brief Get the message field value for the current record
     * \param message A pointer where the field value from the current cursor position will be written
     */
    void Get_message (char message_val[32]) const throw (rdm_exception&);

    /*! \brief Cursor_obj02 Destructor
     *
     * Release the cursor
     */
    ~Cursor_obj02 ();

private:
    friend class Db_obj02_db;
};

}
#endif    /* OBJ02_DB_API_H */
